//
//  ViewController.m
//  Threading
//
//  Created by Jeffrey Ondich on 5/23/14.
//  Copyright (c) 2014 Jeff Ondich. All rights reserved.
//
//  This sample app illustrates the use of Grand Central Dispatch
//  queues for performing background tasks that communicate with the
//  main UI thread (for example, to update a progress bar or report
//  completion of a task).
//

#import "ViewController.h"

@interface ViewController () {
    dispatch_block_t slowOperationBlockIfYouLikeCrashing;
    dispatch_block_t slowOperationBlockIfYouHateCrashing;
    dispatch_queue_t workerQueue;
}

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];

    _progressBar.progress = 0.0;
    _textView.text = @"Hi there";
    
    __weak ViewController *blockSelf = self;
    
    workerQueue = dispatch_queue_create("com.jeffondich.queue.threading_example", NULL);

    slowOperationBlockIfYouLikeCrashing = ^{
        blockSelf.progressBar.progress = 0.0;
        blockSelf.textView.text = [NSString stringWithFormat:@"[%d] Doing important work...", blockSelf.clickCount];
        
        float steps = 5000;
        for (float k = 0; k < steps; k++) {
            blockSelf.progressBar.progress = k / steps;
            for(int j = 0; j < 1000000; j++) {
                // just counting
            }
        }
        blockSelf.clickCount++;
        blockSelf.textView.text = [NSString stringWithFormat:@"[%d] Done!", blockSelf.clickCount];
    };
    
    slowOperationBlockIfYouHateCrashing = ^{
        dispatch_async(dispatch_get_main_queue(), ^{
            blockSelf.progressBar.progress = 0.0;
            blockSelf.textView.text = [NSString stringWithFormat:@"[%d] Doing important work without crashing...", blockSelf.clickCount];
        });

        float steps = 5000;
        for (float k = 0; k < steps; k++) {
            dispatch_async(dispatch_get_main_queue(), ^{
                blockSelf.progressBar.progress = k / steps;
            });
            
            for(int j = 0; j < 1000000; j++) {
                // just counting
            }
        }

        dispatch_async(dispatch_get_main_queue(), ^{
            blockSelf.clickCount++;
            blockSelf.textView.text = [NSString stringWithFormat:@"[%d] Done!", blockSelf.clickCount];
        });
    };
    
    
}

- (IBAction)doSomethingSlowButtonPressed {
//    [self niceTryButNoDice];
    [self thisOneWorks];
}

- (void)niceTryButNoDice {
    dispatch_async(workerQueue, slowOperationBlockIfYouLikeCrashing);
}

- (void)thisOneWorks {
    dispatch_async(workerQueue, slowOperationBlockIfYouHateCrashing);
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
